from django.contrib.auth import get_user_model
from django.contrib.auth import logout
from django.http import JsonResponse
from django.middleware.csrf import get_token
from django.shortcuts import redirect
from rest_framework import status
from rest_framework.authentication import SessionAuthentication, BasicAuthentication
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from sesame.utils import get_query_string

from greg.models import Person


class TokenCreationView(APIView):
    """Token creation endpoint"""

    # Allow anyone to request a new query string to be sent to them
    permission_classes = []

    def _get_username(self, user_model, person: Person) -> str:
        """Find a free username in the database for a person"""
        counter = 1
        while True:
            username = person.first_name[:3] + person.last_name[:3] + str(counter)
            if not user_model.objects.filter(username=username).exists():
                return username
            counter += 1

    def post(self, request, *args, **kwargs):
        """
        Send email to Person with querystring for login

        Persons without a user will have one created for them.
        """
        email = self.kwargs["email"]
        try:
            person = Person.objects.get(email=email)
        except Person.DoesNotExist:
            # Exit if no person with that email (make sure to exit same way as when
            # person does exist to not leak information)
            return Response(status=status.HTTP_200_OK)

        # Create user if person does not have one or fetch existing
        if not person.user:
            user_model = get_user_model()
            username = self._get_username(user_model, person)
            user = user_model.objects.create(username=username)
            person.user = user
            person.save()
        else:
            user = person.user

        # Create querystring and send email
        querystring = get_query_string(user)
        # TODO: send email with query string
        print(querystring)

        return Response(status=status.HTTP_200_OK)


def get_csrf(request):
    response = JsonResponse({"detail": "CSRF cookie set"})
    response["X-CSRFToken"] = get_token(request)
    return response


def logout_view(request):
    if not request.user.is_authenticated:
        return JsonResponse({"detail": "You're not logged in."}, status=400)

    logout(request)
    return JsonResponse({"detail": "Successfully logged out."})


def login_view(request):
    """
    View for pointing login links to

    Sesame will take the query string automatically and use it to create a session for
    the user, so all this needs to do is redirect the user wherever they're supposed to
    go after successfully logging in.
    """
    # TODO: redirect to whatever path the frontend ends up living at (prob '/')
    return redirect("/api/ui/v1/whoami/")


class SessionView(APIView):
    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [IsAuthenticated]

    @staticmethod
    def get(request, format=None):
        return JsonResponse({"isAuthenticated": True})


class WhoAmIView(APIView):
    authentication_classes = [SessionAuthentication, BasicAuthentication]
    permission_classes = [IsAuthenticated]

    @staticmethod
    def get(request, format=None):
        return JsonResponse({"username": request.user.username})
